/*  Pcsx2 - Pc Ps2 Emulator
 *  Copyright (C) 2002-2003  Pcsx2 Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "../PsxCommon.h"
#include "../CDVD.h"
#include "Rpc_services.h"

#define FIXPTR(a)	(a=(void*)((int)a & 0x1FFFFFFF))

int seekpointer=0;//sick pointer;)
/////////////////////////////////////////////// 0001 [OK]
void cdvdfsv5_sceCdRead(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	struct CDREAD *in=(struct CDREAD*)bin;
	struct _rests rests;
	int sector_size;

	if (in->mode.datapattern==CdSecS2328)	sector_size=2328;	else
	if (in->mode.datapattern==CdSecS2340)	sector_size=2340;	else
						sector_size=2048;

#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdRead(lsn=%d, sectors=%d, buf=0x%08X,"
		" mode={trycount=%d, spindlctrl=%d, datapattern=%d, pad=%d},"
		" _rd_intr_data=0x%08X, _Read_cur_pos=0x%08X)\n", 
		in->lsn, in->sectors, in->buf,
		in->mode.trycount, in->mode.spindlctrl, in->mode.datapattern, in->mode.pad, 
		in->_rd_intr_data, in->_Read_cur_pos);
#endif
	//fixup
	FIXPTR(in->buf);
	FIXPTR(in->_rd_intr_data);
	FIXPTR(in->_Read_cur_pos);

	*bout=(CdRead(in->lsn, in->sectors, PSM((int)in->buf), &in->mode)>0);//cool read function;)

	rests.rest.sbuf=in->buf;
	rests.rest.ssize=(int)in->buf & 0xF ? ((int)in->buf & 0xFFFFFFF0) - ((int)in->buf - 16) : 0;
	rests.rest.ebuf=(void*)(((int)in->buf + in->sectors * sector_size) & 0xFFFFFFF0);
	rests.rest.esize=(int)in->buf + in->sectors * sector_size - (int)rests.rest.ebuf;

	memcpy(rests.sbuffer, PSM((int)rests.rest.sbuf), rests.rest.ssize);
	memcpy(rests.ebuffer, PSM((int)rests.rest.ebuf), rests.rest.esize);
	memcpy(PSM((int)in->_rd_intr_data), &rests, sizeof(struct _rests));

	*(int*)(PSM((int)in->_Read_cur_pos)) = in->sectors * sector_size;

	seekpointer=in->lsn+in->sectors;

//	call->rec_id &= ~1;//sceSifCheckStatRpc
	//sceCdSync waits a sema to be reset; but that is done by a callback proc on another thread
}

/////////////////////////////////////////////// 0002
void cdvdfsv5_cdvdman_call40(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] cdvdman_call40() DUMMY\n");
#endif
	*bout=0;
}

/////////////////////////////////////////////// 0003 [OK]
void cdvdfsv5_sceDvdRead(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	struct CDREAD *in=(struct CDREAD*)bin;
	struct _rests rests;
	int sector_size;

	sector_size=2064;

#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceDvdRead(lsn=%d, sectors=%d, buf=0x%08X,"
		" mode={trycount=%d, spindlctrl=%d, datapattern=%d, pad=%d},"
		" _rd_intr_data=0x%08X)\n", 
		in->lsn, in->sectors, in->buf,
		in->mode.trycount, in->mode.spindlctrl, in->mode.datapattern, in->mode.pad, 
		in->_rd_intr_data);
#endif
	//fixup
	FIXPTR(in->buf);
	FIXPTR(in->_rd_intr_data);

	*bout=(DvdRead(in->lsn, in->sectors, PSM((int)in->buf), &in->mode)>0);

	rests.rest.sbuf=in->buf;
	rests.rest.ssize=(int)in->buf & 0xF ? ((int)in->buf & 0xFFFFFFF0) - ((int)in->buf - 16) : 0;
	rests.rest.ebuf=(void*)(((int)in->buf + in->sectors * sector_size) & 0xFFFFFFF0);
	rests.rest.esize=(int)in->buf + in->sectors * sector_size - (int)rests.rest.ebuf;

	memcpy(rests.sbuffer, PSM((int)rests.rest.sbuf), rests.rest.ssize);
	memcpy(rests.ebuffer, PSM((int)rests.rest.ebuf), rests.rest.esize);
	memcpy(PSM((int)in->_rd_intr_data), &rests, sizeof(struct _rests));
	
	seekpointer=in->lsn+in->sectors;

//	call->rec_id &= ~1;//sceSifCheckStatRpc
	//sceCdSync waits a sema to be reset; but that is done by a callback proc on another thread
}

/////////////////////////////////////////////// 0004
void cdvdfsv5_sceCdGetToc(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdGetToc() DUMMY\n");
#endif
	*bout=0;
}

/////////////////////////////////////////////// 0005 [OK]
void cdvdfsv5_sceCdSeek(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdSeek(lsn=0x%08X)\n", *bin);
#endif
	seekpointer=*bin;
	*bout=1;
}

/////////////////////////////////////////////// 0006 [OK]
void cdvdfsv5_sceCdStandby(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdStandby()\n");
#endif
	*bout=1;
}

/////////////////////////////////////////////// 0007 [OK]
void cdvdfsv5_sceCdStop(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdStop()\n");
#endif
	*bout=1;
}

/////////////////////////////////////////////// 0008 [OK]
void cdvdfsv5_sceCdPause(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdPause()\n");
#endif
	*bout=1;
}

/////////////////////////////////////////////// 0009
void cdvdfsv5_sceCdStRead(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdStRead(%X, sect=%X, buf=%X, mode=%X err=%X)\n",
		bin[0], bin[1], bin[2], bin[3], bin[4]);
#endif
	*bout=bin[1];
}

/////////////////////////////////////////////// 000A
void cdvdfsv5_sceCdStReadA(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdStReadA() DUMMY\n");
#endif
	*bout=0;
}

/////////////////////////////////////////////// 000B
void cdvdfsv5_READ_KEY(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] READ_KEY() DUMMY\n");
#endif
	*bout=0;
}

/////////////////////////////////////////////// 000C
void cdvdfsv5_sceCdApplyNCmd(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdApplyNCmd() DUMMY\n");
#endif
	*bout=0;
}

/////////////////////////////////////////////// 000D [not sure; have to check]
void cdvdfsv5_sceCdReadIOPm(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	struct CDREAD *in=(struct CDREAD*)bin;
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdReadIOPm(lsn=%d, sectors=%d, buf=0x%08X,"
		" mode={trycount=%d, spindlctrl=%d, datapattern=%d, pad=%d},"
		" _rd_intr_data=0x%08X, _Read_cur_pos=0x%08X)\n", 
		in->lsn, in->sectors, in->buf,
		in->mode.trycount, in->mode.spindlctrl, in->mode.datapattern, in->mode.pad, 
		in->_rd_intr_data, in->_Read_cur_pos);
#endif
	*bout=(CdRead(in->lsn, in->sectors, PSXM((u32)in->buf), &in->mode)>1);//cool read function;)
	*(int*)(PSM((u32)in->_Read_cur_pos))=in->lsn+in->sectors;//getreadpos=lsn+sectors
}

/////////////////////////////////////////////// 000E [OK]
void cdvdfsv5_sceCdNcmdDiskReady(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdNcmdDiskReady()\n");
#endif
	*bout=2;//SCECdComplete;
}

/////////////////////////////////////////////// 000F
void cdvdfsv5_sceCdReadChain(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
#ifdef RPC_LOG
	RPC_LOG("[RPC:cdvdfsv5] sceCdReadChain() DUMMY\n");
#endif
	*bout=0;
}

//ENTRYPOINT
void sifcall_cdvdfsv5(struct t_sif_rpc_call*	call, u32 *bin, u32 *bout){
	switch (call->rpc_number) {
	case CDVD_READ:		cdvdfsv5_sceCdRead(call, bin, bout);	break;			//OK
	case CDVD_READ2:	cdvdfsv5_cdvdman_call40(call, bin, bout);break;
	case CDVD_READVD:	cdvdfsv5_sceDvdRead(call, bin, bout);	break;			//OK
	case CDVD_GETTOC:	cdvdfsv5_sceCdGetToc(call, bin, bout);	break;
	case CDVD_SEEK:		cdvdfsv5_sceCdSeek(call, bin, bout);	break;			//OK
	case CDVD_STANDBY:	cdvdfsv5_sceCdStandby(call, bin, bout);	break;			//OK
	case CDVD_STOP:		cdvdfsv5_sceCdStop(call, bin, bout);	break;			//OK
	case CDVD_PAUSE:	cdvdfsv5_sceCdPause(call, bin, bout);	break;			//OK
	case CDVD_STREAD:	cdvdfsv5_sceCdStRead(call, bin, bout);	break;
	case CDVD_STREADA:	cdvdfsv5_sceCdStReadA(call, bin, bout);	break;
	case CDVD_READKEY:	cdvdfsv5_READ_KEY(call, bin, bout);		break;
	case CDVD_APPLYNCMD:cdvdfsv5_sceCdApplyNCmd(call, bin, bout);break;
	case CDVD_READIOPM:	cdvdfsv5_sceCdReadIOPm(call, bin, bout);break;			//OK
	case CDVD_NCMDDISKREADY:cdvdfsv5_sceCdNcmdDiskReady(call, bin, bout);break;	//OK
	case CDVD_READCHAIN:cdvdfsv5_sceCdReadChain(call, bin, bout);break;
	default:
#ifdef RPC_LOG
		RPC_LOG("[RPC:cdvdfsv5] <%04X> function is not implemented\n", call->rpc_number);
#endif
		*bout=0;	//failure
	}
}
