/**********************************************************************************
Common sif plugin spec, version #1.0 maintained by FiRES (FiRES-@gmx.de)
***********************************************************************************

Notes:
------

None

**********************************************************************************/
#ifndef _SIF_H_INCLUDED__
#define _SIF_H_INCLUDED__

#if defined(__cplusplus)
extern "C" {
#endif

/* Plugin types */
#define PLUGIN_TYPE_GS    1
#define PLUGIN_TYPE_SIF    2

#define EXPORT      __declspec(dllexport)
#define CALL      _cdecl

/***** Structures *****/
typedef struct {
 WORD Version;        /* Set to 0x0100 */
 WORD Type;           /* Set to PLUGIN_TYPE_SIF */
 char Name[100];      /* Name of the DLL */

} PLUGIN_INFO;


typedef void* (*TGetMemoryPointer)(DWORD _dwAddress);
typedef void (*TSignalSema)(int _dwSemaId);
typedef void (*TLog)(char* _pMessage);

typedef void (*TCallFunction)(struct t_sif_rpc_call* _pRPCCall,
unsigned __int32* _pIn, unsigned __int32* _pOut);

typedef struct 
{
 DWORD   dwBindNumber;  /* Bindnumber (eg: 0x80000006 = MODLOAD)
*/
 TCallFunction pCallFunction;  /* pointer to the call function
*/
} SModuleBind;


typedef struct
{
 TGetMemoryPointer pGetMemoryPointer;
 TSignalSema   pSignalSema;
 TLog    pLog;
} SSifInitialize;

/******************************************************************
  Function: GetDllInfo
  Purpose:  This function allows the emulator to gather information
            about the dll by filling in the PluginInfo structure.
  input:    a pointer to a PLUGIN_INFO stucture that needs to
be
            filled by the function. (see def above)
  output:   none
*******************************************************************/
void CALLBACK GetDllInfo (PLUGIN_INFO * _PluginInfo);

/******************************************************************
  Function: CloseDLL
  Purpose:  This function is called when the emulator is closing
            down allowing the dll to de-initialise.
  input:    none
  output:   none
*******************************************************************/
void CALLBACK CloseDLL (void);

/******************************************************************
  Function: DllAbout
  Purpose:  This function is optional function that is provided
            to give further information about the DLL.
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/
void CALLBACK DllAbout (HWND _hParent);

/******************************************************************
  Function: DllConfig
  Purpose:  This function is optional function that is provided
            to allow the user to configure the dll
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/
void CALLBACK DllConfig (HWND _hParent);

/******************************************************************
  Function: DllTest
  Purpose:  This function is optional function that is provided
            to allow the user to test the dll
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/
void CALLBACK DllTest (HWND _hParent);

/******************************************************************
  Function: 
  Purpose:  
  input:    SSifInitialize
  output:   none
*******************************************************************/
void CALLBACK Initialize (SSifInitialize _SifInitialize);

/******************************************************************
  Function: BindModule
  Purpose:  Function to bind a Module Function
  input:    ModuleBind-Structur
  output:   true if the bind is successfull
*******************************************************************/
BOOL CALLBACK BindModule (SModuleBind* _pModuleBind); 

/******************************************************************
  Function: SetRPCVersion
  Purpose:  Set the RPCVersion
  input:    RPCVersion
  output:   bone
*******************************************************************/
void CALLBACK SetRPCVersion (char* _ioprp); 

int CALLBACK Run_DVD ( void );

#if defined(__cplusplus)
}
#endif
#endif

